import json

from jirafs.exceptions import JirafsError
from jirafs.plugin import CommandPlugin


class Command(CommandPlugin):
    """ Get the status of the current ticketfolder """

    TRY_SUBFOLDERS = True
    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def handle(self, args, folder, **kwargs):
        return self.cmd(folder, args.field_name, args.value, as_json=args.json)

    def add_arguments(self, parser):
        parser.add_argument("field_name",)
        parser.add_argument("value",)
        parser.add_argument(
            "--json",
            help=("Interpret value argument as JSON data."),
            action="store_true",
            default=False,
        )

    def main(self, folder, field_name, value, as_json=False):
        fields = folder.get_fields()

        if as_json:
            try:
                value = json.loads(value)
            except ValueError:
                raise JirafsError("Value '%s' could not be decoded as JSON." % (value,))

        key_dotpath = None
        if "." in field_name:
            field_name, key_dotpath = field_name.split(".", 1)

        if field_name not in fields:
            raise JirafsError("Field '%s' does not exist." % field_name)

        if key_dotpath:
            data = fields[field_name]
            try:
                cursor = data
                dotpath_parts = key_dotpath.split(".")
                last_key = len(dotpath_parts) - 1
                for idx, component in enumerate(dotpath_parts):
                    if idx == last_key:
                        cursor[component] = value
                        break
                    elif not isinstance(cursor.get(component), dict):
                        raise JirafsError(
                            "Key '%s' (of dotpath '%s') is not an object "
                            "in field '%s'." % (component, key_dotpath, field_name,)
                        )
                    else:
                        if component not in data:
                            raise JirafsError(
                                "Key '%s' (of dotpath '%s') could not be found "
                                "in field '%s'." % (component, key_dotpath, field_name,)
                            )
                        cursor = cursor[component]
            except (ValueError, TypeError):
                raise JirafsError(
                    "Field '%s' could not be parsed as JSON for retrieving "
                    "dotpath '%s'." % (field_name, key_dotpath,)
                )
            value = data
        else:
            data = value

        fields[field_name] = data
        fields.write()



from collections import OrderedDict

from jirafs.plugin import CommandPlugin
from jirafs.utils import run_command_method_with_kwargs
from jirafs.exceptions import JiraInteractionFailed


class Command(CommandPlugin):
    """ Transition the current issue into a new state """

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def handle(self, args, folder, **kwargs):
        state = self.get_state_from_string(folder, args.state)
        if state is None:
            state = self.get_state_from_user(folder)
        return self.cmd(folder, state)

    def main(self, folder, state_id):
        folder.jira.transition_issue(folder.issue, state_id)
        starting_status = folder.get_fields()["status"]
        pull_result = run_command_method_with_kwargs("pull", folder=folder)

        if starting_status == folder.get_fields()["status"]:
            # I'd love it if we could instead just check the response code
            # from the transitions API, but that API returns a 204 whether
            # or not the issue itself can be successfully transitioned.
            raise JiraInteractionFailed(
                "JIRA was not able to successfully transition this issue "
                "into the requested state.  This type of failure usually "
                "occurs when one's JIRA configuration requires that certain "
                "fields be specified before transitioning into a "
                "given state.  Unfortunately, no details regarding what "
                "fields may be required are provided via JIRA's API."
            )

        return pull_result[1]

    def get_transition_dict(self, folder):
        if not hasattr(self, "_transition_dict"):
            value = folder.jira.transitions(folder.issue)
            self._transition_dict = OrderedDict((v["id"], v) for v in value)
        return self._transition_dict

    def get_state_from_string(self, folder, value):
        options = self.get_transition_dict(folder)

        if value is None:
            return None

        # If it's a dictionary key, just go for it.
        if value in options:
            return value

        # But, if it isn't, this'll be a bit less efficient
        for k, v in options.items():
            if value.upper() == v["name"].upper():
                return k

        return None

    def get_state_from_user(self, folder):
        options = self.get_transition_dict(folder)
        response = None
        while response is None:
            for option_id, option_data in options.items():
                print(
                    "%s: %s (%s)"
                    % (
                        option_id,
                        option_data["name"],
                        option_data.get("to", {}).get("description", ""),
                    )
                )

            print("")
            response = self.get_state_from_string(
                folder, input("Please select a state from the above options: ")
            )

        return response

    def add_arguments(self, parser):
        parser.add_argument("state", default=None, nargs="?")
